"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const for_each_1 = require("../async/for-each");
const directory_reader_1 = require("../directory-reader");
const iteratorFacade = { fs, forEach: for_each_1.asyncForEach };
const iterator = readdirIterator;
exports.iterator = iterator;
iterator.stat = readdirIteratorStat;
/**
 * Aynchronous `readdir()` that returns an `AsyncIterableIterator` (an object that implements
 * both the `AsyncIterable` and `AsyncIterator` interfaces) that yields path strings.
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Iterators_and_Generators
 */
function readdirIterator(dir, options) {
    return readdirIteratorInternal(dir, options, {});
}
exports.readdirIterator = readdirIterator;
/**
 * Aynchronous `readdir()` that returns an `AsyncIterableIterator` (an object that implements
 * both the `AsyncIterable` and `AsyncIterator` interfaces) that yields `Stats` objects.
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Iterators_and_Generators
 */
function readdirIteratorStat(dir, options) {
    return readdirIteratorInternal(dir, options, { stats: true });
}
exports.readdirIteratorStat = readdirIteratorStat;
/**
 * Yields each chunk of an asynchronous `DirectoryReader`.
 */
function readdirIteratorInternal(dir, options, behavior) {
    let reader = new directory_reader_1.DirectoryReader(dir, options, behavior, iteratorFacade);
    let stream = reader.stream;
    let error;
    stream.on("error", (err) => {
        error = err;
        stream.pause();
    });
    let done = false;
    stream.on("end", () => {
        done = true;
    });
    return {
        [Symbol.asyncIterator]() {
            return this;
        },
        // tslint:disable-next-line: promise-function-async
        next() {
            return new Promise((resolve, reject) => {
                shortCircuit() || stream.on("readable", readNextResult);
                function shortCircuit() {
                    if (error) {
                        reject(error);
                        return true;
                    }
                    else if (done) {
                        resolve({ done, value: undefined });
                        return true;
                    }
                }
                function readNextResult() {
                    try {
                        if (shortCircuit()) {
                            return;
                        }
                        let value = stream.read();
                        stream.off("readable", readNextResult);
                        if (value === null) {
                            done = true;
                        }
                        shortCircuit() || resolve({ value: value });
                    }
                    catch (err) {
                        reject(err);
                    }
                }
            });
        },
    };
}
//# sourceMappingURL=index.js.map